# UNICADO - UNIversity Conceptual Aircraft Design and Optimization
#
# Copyright (C) 2025 UNICADO consortium
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.
#
# Description:
# This file is part of UNICADO.

def path_check(path_to_check):
    """ Function to check if the given input path an absolute or relative path and correct the given path.

        The input string 'path_to_check' contains the string to check if it is an absolute or relative path.

        The output string 'corrected_path' contains the corrected path.

        :param: path_to_check: input string
        :return: path_to_check_is_absolute: output bool, corrected_path: output string
    """
    ''' imports for python '''
    import os
    
    # check if the given path starts with '..'
    #  -> if true: -> cut the first to characters and set the correct absolute path
    if path_to_check[:2] == '..':
        corrected_path = path_to_check[3:]
    # else condition: input path is an absolut path
    else:
        corrected_path = path_to_check

    # replace all '\' character with '/' to operate on Windows and linux OS
    corrected_path = corrected_path.replace(os.sep, '/')
    
    # check if corrected_path is an absolute path
    path_to_check_is_absolute = os.path.isabs(path_to_check)

    # check if the last character of the corrected_path string is a '/'
    #  -> if not: -> add '/' to the end of corrected_path string
    if not corrected_path[-1] == '/':
        corrected_path = corrected_path + '/'

    return path_to_check_is_absolute, corrected_path
