# UNICADO - UNIversity Conceptual Aircraft Design and Optimization
#
# Copyright (C) 2025 UNICADO consortium
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.
#
# Description:
# This file is part of UNICADO.

def read_geometry_elements(root_of_aircraft_exchange_file):
    """ reads the used geometry elements from the aircraft exchange file

        The input dictionary contains the element tree of the aircraft exchange file of current project.

        The input list contains the geometry data file names of the used elements of current project.

        :param: root_of_aircraft_exchange_file: input dictionary
        :return: geometry_name: output list
    """

    # initialize used geometry name list as empty list
    geometry_name = set([])

    ''' read horizontal lifting surfaces geometry data '''
    aerodynamic_surfaces = root_of_aircraft_exchange_file.findall(
                "./component_design/wing/specific/geometry/aerodynamic_surface")
    aerodynamic_surfaces.extend(root_of_aircraft_exchange_file.findall(
                "./component_design/empennage/specific/geometry/aerodynamic_surface"))
    # loop across all used aerodynamic surface elements
    for aerodynamic_surface in aerodynamic_surfaces:
        # loop across all half surface profiles for symmetrical and unsymmetrical lifting surfaces
        for half_surface_segment_profile in aerodynamic_surface.findall("./parameters/sections/section/profile"):
            geometry_name.add(str(half_surface_segment_profile.find("./value").text))

    ''' read fuselage geometry data '''
    fuselage_shapes = root_of_aircraft_exchange_file.findall(
        "./component_design/fuselage/specific/geometry/fuselage/sections/section/section_shape")

    # loop across all used fuselage shape elements surface elements
    for fuselage_shape in fuselage_shapes:
        # add element if it is a .dat file
        shape_name = str(fuselage_shape.find("./value").text)
        print (shape_name[-4:])
        if shape_name[-4:] in ".dat":
            geometry_name.add(shape_name[:-4])

    ''' read nacelle geometry data '''
    nacelle_profiles = root_of_aircraft_exchange_file.findall(
            "./component_design/propulsion/specific/propulsion/nacelle/sections/section/profile")

    # loop across all used nacelle profile elements
    for nacelle_profile in nacelle_profiles:
        geometry_name.add(str(nacelle_profile.find("./value").text))

    ''' read pylon geometry data '''
    pylon_profiles = root_of_aircraft_exchange_file.findall(
            "./component_design/propulsion/specific/propulsion/pylon/sections/section/profile")

    # loop across all used pylon profile elements
    for pylon_profile in pylon_profiles:
        geometry_name.add(str(pylon_profile.find("./value").text))

    return geometry_name
