# UNICADO - UNIversity Conceptual Aircraft Design and Optimization
#
# Copyright (C) 2025 UNICADO consortium
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.
#
# Description:
# This file is part of UNICADO.

def xml_path_check(xml_path_to_check):
    """ Function to check if the given input xml-path an absolute or relative xml-path and correct the given path.

        The input string 'xml_path_to_check' contains the string to check if it is an absolute or relative xml-path.

        The output string 'corrected_path' contains the corrected path.

        :param: xml_path_to_check: input string
        :return: corrected_xml_path: output string
    """

    ''' imports for python '''
    import os

    # replace all '\' character with '/' to operate on Windows and linux OS
    xml_path_to_check = xml_path_to_check.replace(os.sep, '/')

    # check if the string "AcftExchangeFile" in the given xml-path -> if true: -> delete from string
    if "aircraft_exchange_file" in xml_path_to_check:
        length_of_string = len("aircraft_exchange_file")
        index = xml_path_to_check.find('aircraft_exchange_file')
        corrected_xml_path = '.' + xml_path_to_check[index+length_of_string:]

    # check if the string "ConfigFile" in the given xml-path -> if true: -> delete from string
    elif "module_configuration_file" in xml_path_to_check:
        length_of_string = len("module_configuration_file")
        index = xml_path_to_check.find('module_configuration_file')
        corrected_xml_path = '.' + xml_path_to_check[index+length_of_string:]

    else:
        if xml_path_to_check[0] == '/':
            corrected_xml_path = '.' + xml_path_to_check

        elif not xml_path_to_check[0:1] == './':
            if xml_path_to_check[0] == '.':
                corrected_xml_path = './' + xml_path_to_check[1:]
            else:
                corrected_xml_path = './' + xml_path_to_check
        else:
            corrected_xml_path = xml_path_to_check

    return corrected_xml_path
